#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>


/*************************************************************
 * java_fser_
 * c program to link with fortran 90
 * This program writes the x and fx values to file.
 *
 * input parameters
 *   name - name of the file
 *   time - time value
 *   x    - x value array
 *   fx   - fx value array
 *   nx   - size of x and fx array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 ************************************************************/

int java_fser_(name, time, x, fx, nx) 
  char *name;
  double *time;
  double x[];
  double fx[];
  int *nx;
{
   int size, i;
   FILE *fopen(), *fp;

   if ((fp = fopen(name, "a+b")) == NULL) {
      printf("Error: can't open output file ...\n");
      return 1;
   }

   /* first write time and nx */
   fwrite(time, sizeof(double), 1, fp);
   fwrite(nx, sizeof(int), 1, fp);

   size = *nx;
   
   /* write x and fx value */
   for (i=0; i<size; i++) {
      fwrite(&x[i], sizeof(double), 1, fp);
      fwrite(&fx[i], sizeof(double), 1, fp);
   }
   
   fclose(fp);
   return 0;
}


/*************************************************************
 * java_ser_
 * c program to link with fortran 90
 * This program writes the x and fx values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   fx   - fx value array
 *   nx   - size of x and fx array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_ser_(name, time, x, fx, nx) 
  char *name;
  double *time;
  double x[];
  double fx[];
  int *nx;
{
   int sockfd;
   int size, name_length, i;

   char *hostname;
   char *rawport;
   int port;

   int command = 10;
   struct sockaddr_in serv_addr;

   /** 
    * first read system environment JSERHOST and JSERPORT
    */
   hostname = getenv("JSERHOST");
   rawport = getenv("JSERPORT");

   if (hostname == NULL) {
      printf("Unknown server host name error ...\n");
      return 1;
   }
   
   if (rawport == NULL) 
      port = 7654;
   else
      port = atoi(rawport);

   /*
    * fill in the structure "serv_addr" with the address 
    * of the server that we want to connect with 
    */
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   serv_addr.sin_addr.s_addr = inet_addr(hostname);
   serv_addr.sin_port = htons(port);

   /*
    * open a TCP socket (an Internet stream socket).
    */
   if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      printf("Can't open stream socket.\n");
      return 1;
   }


   /*
    * connect to server
    */
   if (connect(sockfd, (struct sockaddr *) &serv_addr, 
                                    sizeof(serv_addr)) < 0) {
      printf("Can't connect to server.\n");
      return 1;
   }


   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);


   size = *nx; 
   /* write time and nx */
   write(sockfd, time, sizeof(double));
   write(sockfd, &size, sizeof(int));

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write(sockfd, &x[i], sizeof(double));
      write(sockfd, &fx[i], sizeof(double));
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_mser_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x and fx values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   fx   - fx value array
 *   nx   - size of x and fx array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_mser_(name, nt, time, x, fx, nx) 
  char *name;
  int *nt;
  double time[];
  double x[];
  double fx[];
  int nx[];
{
   int sockfd;
   int ntsize, nxsize, name_length, i, j, index;

   char *hostname;
   char *rawport;
   int port;

   int command = 12;
   struct sockaddr_in serv_addr;

   /** 
    * first read system environment JSERHOST and JSERPORT
    */
   hostname = getenv("JSERHOST");
   rawport = getenv("JSERPORT");

   if (hostname == NULL) {
      printf("Unknown server host name error ...\n");
      return 1;
   }
   
   if (rawport == NULL) 
      port = 7654;
   else
      port = atoi(rawport);

   /*
    * fill in the structure "serv_addr" with the address 
    * of the server that we want to connect with 
    */
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   serv_addr.sin_addr.s_addr = inet_addr(hostname);
   serv_addr.sin_port = htons(port);

   /*
    * open a TCP socket (an Internet stream socket).
    */
   if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      printf("Can't open stream socket.\n");
      return 1;
   }


   /*
    * connect to server
    */
   if (connect(sockfd, (struct sockaddr *) &serv_addr, 
                                    sizeof(serv_addr)) < 0) {
      printf("Can't connect to server.\n");
      return 1;
   }


   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);


   ntsize = *nt; 
   /* send nt */
   write(sockfd, &ntsize, sizeof(int));

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write(sockfd, &time[i], sizeof(double));

      nxsize = nx[i];     
      write(sockfd, &nxsize, sizeof(int));
      for (j=0; j<nxsize; j++) {
         write(sockfd, &x[index], sizeof(double));
         write(sockfd, &fx[index], sizeof(double));
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}




/*************************************************************
 * java_fser2d_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to file.
 *
 * input parameters
 *   name - name of the file
 *   time - time value
 *   x    - x value array
 *   y    - y value array
 *   fxy  - fxy value array
 *   nx   - size of x, y and fx array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 ************************************************************/

int java_fser2d_(name, time, x, y, fxy, nx) 
  char *name;
  double *time;
  double x[];
  double y[];
  double fxy[];
  int *nx;
{
   int size, i;
   FILE *fopen(), *fp;

   if ((fp = fopen(name, "a+b")) == NULL) {
      printf("Error: can't open output file ...\n");
      return 1;
   }

   /* first write time and nx */
   fwrite(time, sizeof(double), 1, fp);
   fwrite(nx, sizeof(int), 1, fp);

   size = *nx;
   
   /* write x and fx value */
   for (i=0; i<size; i++) {
      fwrite(&x[i], sizeof(double), 1, fp);
      fwrite(&y[i], sizeof(double), 1, fp);
      fwrite(&fxy[i], sizeof(double), 1, fp);
   }
   
   fclose(fp);
   return 0;
}


/*************************************************************
 * java_ser2d_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   y    - y value array
 *   fxy  - fxy value array
 *   nx   - size of x and fx array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_ser2d_(name, time, x, y, fxy, nx) 
  char *name;
  double *time;
  double x[];
  double y[];
  double fxy[];
  int *nx;
{
   int sockfd;
   int size, name_length, i;

   char *hostname;
   char *rawport;
   int port;

   int command = 15;
   struct sockaddr_in serv_addr;

   /** 
    * first read system environment JSERHOST and JSERPORT
    */
   hostname = getenv("JSERHOST");
   rawport = getenv("JSERPORT");

   if (hostname == NULL) {
      printf("Unknown server host name error ...\n");
      return 1;
   }
   
   if (rawport == NULL) 
      port = 7654;
   else
      port = atoi(rawport);

   /*
    * fill in the structure "serv_addr" with the address 
    * of the server that we want to connect with 
    */
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   serv_addr.sin_addr.s_addr = inet_addr(hostname);
   serv_addr.sin_port = htons(port);

   /*
    * open a TCP socket (an Internet stream socket).
    */
   if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      printf("Can't open stream socket.\n");
      return 1;
   }


   /*
    * connect to server
    */
   if (connect(sockfd, (struct sockaddr *) &serv_addr, 
                                    sizeof(serv_addr)) < 0) {
      printf("Can't connect to server.\n");
      return 1;
   }


   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);


   size = *nx; 
   /* write time and nx */
   write(sockfd, time, sizeof(double));
   write(sockfd, &size, sizeof(int));

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write(sockfd, &x[i], sizeof(double));
      write(sockfd, &y[i], sizeof(double));
      write(sockfd, &fxy[i], sizeof(double));
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_mser2d_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x, y and fxy values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   y    - y value array
 *   fxy   - fxy value array
 *   nx   - size of x and fx array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_mser2d_(name, nt, time, x, y, fxy, nx) 
  char *name;
  int *nt;
  double time[];
  double x[];
  double y[];
  double fxy[];
  int nx[];
{
   int sockfd;
   int ntsize, nxsize, name_length, i, j, index;

   char *hostname;
   char *rawport;
   int port;

   int command = 16;
   struct sockaddr_in serv_addr;

   /** 
    * first read system environment JSERHOST and JSERPORT
    */
   hostname = getenv("JSERHOST");
   rawport = getenv("JSERPORT");

   if (hostname == NULL) {
      printf("Unknown server host name error ...\n");
      return 1;
   }
   
   if (rawport == NULL) 
      port = 7654;
   else
      port = atoi(rawport);

   /*
    * fill in the structure "serv_addr" with the address 
    * of the server that we want to connect with 
    */
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   serv_addr.sin_addr.s_addr = inet_addr(hostname);
   serv_addr.sin_port = htons(port);

   /*
    * open a TCP socket (an Internet stream socket).
    */
   if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      printf("Can't open stream socket.\n");
      return 1;
   }


   /*
    * connect to server
    */
   if (connect(sockfd, (struct sockaddr *) &serv_addr, 
                                    sizeof(serv_addr)) < 0) {
      printf("Can't connect to server.\n");
      return 1;
   }


   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);


   ntsize = *nt; 
   /* send nt */
   write(sockfd, &ntsize, sizeof(int));

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write(sockfd, &time[i], sizeof(double));

      nxsize = nx[i];     
      write(sockfd, &nxsize, sizeof(int));
      for (j=0; j<nxsize; j++) {
         write(sockfd, &x[index], sizeof(double));
         write(sockfd, &y[index], sizeof(double));
         write(sockfd, &fxy[index], sizeof(double));
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}



/*************************************************************
 * java_bbser2d_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   xmin    - x min value
 *   xmax    - x max value
 *   nx      - number of x points 
 *   ymin    - y min value
 *   ymax    - y max value
 *   ny      - number of y points 
 *   fxy     - fxy value array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_bbser2d_(name, time, xmin, xmax, nx, ymin, ymax, ny, fxy) 
  char *name;
  double *time;
  double *xmin;
  double *xmax;
  int *nx;
  double *ymin;
  double *ymax;
  int *ny;
  double fxy[];
{
   int sockfd;
   int size, xsize, ysize, name_length, i;

   char *hostname;
   char *rawport;
   int port;

   int command = 25;
   struct sockaddr_in serv_addr;

   /** 
    * first read system environment JSERHOST and JSERPORT
    */
   hostname = getenv("JSERHOST");
   rawport = getenv("JSERPORT");

   if (hostname == NULL) {
      printf("Unknown server host name error ...\n");
      return 1;
   }
   
   if (rawport == NULL) 
      port = 7654;
   else
      port = atoi(rawport);

   /*
    * fill in the structure "serv_addr" with the address 
    * of the server that we want to connect with 
    */
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   serv_addr.sin_addr.s_addr = inet_addr(hostname);
   serv_addr.sin_port = htons(port);

   /*
    * open a TCP socket (an Internet stream socket).
    */
   if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      printf("Can't open stream socket.\n");
      return 1;
   }


   /*
    * connect to server
    */
   if (connect(sockfd, (struct sockaddr *) &serv_addr, 
                                    sizeof(serv_addr)) < 0) {
      printf("Can't connect to server.\n");
      return 1;
   }


   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);

   xsize = *nx;
   ysize = *ny;
   size = xsize * ysize;
   /* write time and nx */
   write(sockfd, time, sizeof(double));
   write(sockfd, nx, sizeof(int));
   write(sockfd, ny, sizeof(int));
   write(sockfd, xmin, sizeof(double));
   write(sockfd, xmax, sizeof(double));
   write(sockfd, ymin, sizeof(double));
   write(sockfd, ymax, sizeof(double));

   /* write fx value */
   for (i=0; i<size; i++) {
      write(sockfd, &fxy[i], sizeof(double));
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_bbmser2d_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x, y and fxy values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   xmin    - x min value array
 *   xmax    - x max value array
 *   nx    -   nx value array
 *   ymin    - y min value array
 *   ymax    - y max value array
 *   ny    -   ny value array
 *   fxy   - fxy value array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_bbmser2d_(name, nt, time, xmin, xmax, nx, ymin, ymax, ny, fxy) 
  char *name;
  int *nt;
  double time[];
  double xmin[];
  double xmax[];
  int nx[];
  double ymin[];
  double ymax[];
  int ny[];
  double fxy[];

{
   int sockfd;
   int ntsize, nxsize, nysize, size, name_length, i, j, index;

   char *hostname;
   char *rawport;
   int port;

   int command = 26;
   struct sockaddr_in serv_addr;

   /** 
    * first read system environment JSERHOST and JSERPORT
    */
   hostname = getenv("JSERHOST");
   rawport = getenv("JSERPORT");

   if (hostname == NULL) {
      printf("Unknown server host name error ...\n");
      return 1;
   }
   
   if (rawport == NULL) 
      port = 7654;
   else
      port = atoi(rawport);

   /*
    * fill in the structure "serv_addr" with the address 
    * of the server that we want to connect with 
    */
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   serv_addr.sin_addr.s_addr = inet_addr(hostname);
   serv_addr.sin_port = htons(port);

   /*
    * open a TCP socket (an Internet stream socket).
    */
   if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      printf("Can't open stream socket.\n");
      return 1;
   }


   /*
    * connect to server
    */
   if (connect(sockfd, (struct sockaddr *) &serv_addr, 
                                    sizeof(serv_addr)) < 0) {
      printf("Can't connect to server.\n");
      return 1;
   }


   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);


   ntsize = *nt; 
   /* send nt */
   write(sockfd, &ntsize, sizeof(int));

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write(sockfd, &time[i], sizeof(double));

      nxsize = nx[i];   
      nysize = ny[i];  
      write(sockfd, &nxsize, sizeof(int));
      write(sockfd, &nysize, sizeof(int));
      write(sockfd, &xmin[i], sizeof(double));
      write(sockfd, &xmax[i], sizeof(double));
      write(sockfd, &ymin[i], sizeof(double));
      write(sockfd, &ymax[i], sizeof(double));
     
      size = nxsize * nysize;
      for (j=0; j<size; j++) {
         write(sockfd, &fxy[index], sizeof(double));
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}




/*************************************************************
 * java_vser_
 * c program to link with fortran 90
 * This program writes the vector data to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   y    - y value array
 *   r  - r value array
 *   t  - r value array
 *   nx   - size of x, y, r, t array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_vser_(name, time, x, y, r, t, nx) 
  char *name;
  double *time;
  double x[];
  double y[];
  double r[];
  double t[];
  int *nx;
{
   int sockfd;
   int size, name_length, i;

   char *hostname;
   char *rawport;
   int port;

   int command = 22;
   struct sockaddr_in serv_addr;

   /** 
    * first read system environment JSERHOST and JSERPORT
    */
   hostname = getenv("JSERHOST");
   rawport = getenv("JSERPORT");

   if (hostname == NULL) {
      printf("Unknown server host name error ...\n");
      return 1;
   }
   
   if (rawport == NULL) 
      port = 7654;
   else
      port = atoi(rawport);

   /*
    * fill in the structure "serv_addr" with the address 
    * of the server that we want to connect with 
    */
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   serv_addr.sin_addr.s_addr = inet_addr(hostname);
   serv_addr.sin_port = htons(port);

   /*
    * open a TCP socket (an Internet stream socket).
    */
   if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      printf("Can't open stream socket.\n");
      return 1;
   }


   /*
    * connect to server
    */
   if (connect(sockfd, (struct sockaddr *) &serv_addr, 
                                    sizeof(serv_addr)) < 0) {
      printf("Can't connect to server.\n");
      return 1;
   }


   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);


   size = *nx; 
   /* write time and nx */
   write(sockfd, time, sizeof(double));
   write(sockfd, &size, sizeof(int));

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write(sockfd, &x[i], sizeof(double));
      write(sockfd, &y[i], sizeof(double));
      write(sockfd, &r[i], sizeof(double));
      write(sockfd, &t[i], sizeof(double));
   }
   
   close(sockfd);
   return 0;
}




/*************************************************************
 * java_vmser_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x and fx values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   y    - y value array
 *   r  - r value array
 *   t  - t value array
 *   nx   - size of x, y, r, t array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_vmser_(name, nt, time, x, y, r, t, nx) 
  char *name;
  int *nt;
  double time[];
  double x[];
  double y[];
  double r[];
  double t[];
  int nx[];
{
   int sockfd;
   int ntsize, nxsize, name_length, i, j, index;

   char *hostname;
   char *rawport;
   int port;

   int command = 27;
   struct sockaddr_in serv_addr;

   /** 
    * first read system environment JSERHOST and JSERPORT
    */
   hostname = getenv("JSERHOST");
   rawport = getenv("JSERPORT");

   if (hostname == NULL) {
      printf("Unknown server host name error ...\n");
      return 1;
   }
   
   if (rawport == NULL) 
      port = 7654;
   else
      port = atoi(rawport);

   /*
    * fill in the structure "serv_addr" with the address 
    * of the server that we want to connect with 
    */
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   serv_addr.sin_addr.s_addr = inet_addr(hostname);
   serv_addr.sin_port = htons(port);

   /*
    * open a TCP socket (an Internet stream socket).
    */
   if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      printf("Can't open stream socket.\n");
      return 1;
   }


   /*
    * connect to server
    */
   if (connect(sockfd, (struct sockaddr *) &serv_addr, 
                                    sizeof(serv_addr)) < 0) {
      printf("Can't connect to server.\n");
      return 1;
   }


   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);


   ntsize = *nt; 
   /* send nt */
   write(sockfd, &ntsize, sizeof(int));

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write(sockfd, &time[i], sizeof(double));

      nxsize = nx[i];     
      write(sockfd, &nxsize, sizeof(int));
      for (j=0; j<nxsize; j++) {
         write(sockfd, &x[index], sizeof(double));
         write(sockfd, &y[index], sizeof(double));
         write(sockfd, &r[index], sizeof(double));
         write(sockfd, &t[index], sizeof(double));
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}




/*************************************************************
 * java_bbser_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   xmin    - x min value
 *   xmax    - x max value
 *   nx      - number of x points 
 *   fx     - fxy value array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_bbser_(name, time, xmin, xmax, nx, fx) 
  char *name;
  double *time;
  double *xmin;
  double *xmax;
  int *nx;
  double fx[];
{
   int sockfd;
   int size, name_length, i;

   char *hostname;
   char *rawport;
   int port;

   int command = 28;
   struct sockaddr_in serv_addr;

   /** 
    * first read system environment JSERHOST and JSERPORT
    */
   hostname = getenv("JSERHOST");
   rawport = getenv("JSERPORT");

   if (hostname == NULL) {
      printf("Unknown server host name error ...\n");
      return 1;
   }
   
   if (rawport == NULL) 
      port = 7654;
   else
      port = atoi(rawport);

   /*
    * fill in the structure "serv_addr" with the address 
    * of the server that we want to connect with 
    */
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   serv_addr.sin_addr.s_addr = inet_addr(hostname);
   serv_addr.sin_port = htons(port);

   /*
    * open a TCP socket (an Internet stream socket).
    */
   if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      printf("Can't open stream socket.\n");
      return 1;
   }


   /*
    * connect to server
    */
   if (connect(sockfd, (struct sockaddr *) &serv_addr, 
                                    sizeof(serv_addr)) < 0) {
      printf("Can't connect to server.\n");
      return 1;
   }


   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);

   size = *nx;

   /* write time and nx */
   write(sockfd, time, sizeof(double));
   write(sockfd, nx, sizeof(int));
   write(sockfd, xmin, sizeof(double));
   write(sockfd, xmax, sizeof(double));


   /* write fx value */
   for (i=0; i<size; i++) {
      write(sockfd, &fx[i], sizeof(double));
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_bbmser_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x, y and fxy values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   xmin    - x min value array
 *   xmax    - x max value array
 *   nx    -   nx value array
 *   fx   - fxy value array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_bbmser_(name, nt, time, xmin, xmax, nx, fx) 
  char *name;
  int *nt;
  double time[];
  double xmin[];
  double xmax[];
  int nx[];
  double fx[];

{
   int sockfd;
   int ntsize, size, name_length, i, j, index;

   char *hostname;
   char *rawport;
   int port;

   int command = 26;
   struct sockaddr_in serv_addr;

   /** 
    * first read system environment JSERHOST and JSERPORT
    */
   hostname = getenv("JSERHOST");
   rawport = getenv("JSERPORT");

   if (hostname == NULL) {
      printf("Unknown server host name error ...\n");
      return 1;
   }
   
   if (rawport == NULL) 
      port = 7654;
   else
      port = atoi(rawport);

   /*
    * fill in the structure "serv_addr" with the address 
    * of the server that we want to connect with 
    */
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   serv_addr.sin_addr.s_addr = inet_addr(hostname);
   serv_addr.sin_port = htons(port);

   /*
    * open a TCP socket (an Internet stream socket).
    */
   if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      printf("Can't open stream socket.\n");
      return 1;
   }


   /*
    * connect to server
    */
   if (connect(sockfd, (struct sockaddr *) &serv_addr, 
                                    sizeof(serv_addr)) < 0) {
      printf("Can't connect to server.\n");
      return 1;
   }


   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);


   ntsize = *nt; 
   /* send nt */
   write(sockfd, &ntsize, sizeof(int));

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write(sockfd, &time[i], sizeof(double));

      size = nx[i];   
      write(sockfd, &size, sizeof(int));
      write(sockfd, &xmin[i], sizeof(double));
      write(sockfd, &xmax[i], sizeof(double));
     
      for (j=0; j<size; j++) {
         write(sockfd, &fx[index], sizeof(double));
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}

